import { Component, OnInit, Inject, OnDestroy } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material';
import { KEY_LOGIN_RES_INFO } from '@app/types/login-res-info.type';
import { KEY_VER_INFO } from '@app/types/ver-info.type';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';

import { Store } from '@ngrx/store';

import { DialogService } from '@ucap-webmessenger/ui';
import { VersionInfo2Response } from '@ucap-webmessenger/api-public';
import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { AppAuthenticationService } from '@app/services/authentication.service';
import { StringUtil, CharactorType } from '@ucap-webmessenger/core';

import * as ServiceStore from '@app/store/messenger/service';
import CryptoJS from 'crypto-js';

export interface ChangePasswordDialogData {}

export interface ChangePasswordDialogResult {
  choice: boolean;
}

@Component({
  selector: 'app-change-password.dialog',
  templateUrl: './change-password.dialog.component.html',
  styleUrls: ['./change-password.dialog.component.scss']
})
export class ChangePasswordDialogComponent implements OnInit, OnDestroy {
  loginRes: LoginResponse;
  sessionVerinfo: VersionInfo2Response;
  isMe: boolean;

  constructor(
    public dialogRef: MatDialogRef<
      ChangePasswordDialogData,
      ChangePasswordDialogResult
    >,
    @Inject(MAT_DIALOG_DATA) public data: ChangePasswordDialogData,
    private dialogService: DialogService,
    private sessionStorageService: SessionStorageService,
    private appAuthenticationService: AppAuthenticationService,
    private store: Store<any>
  ) {
    this.sessionVerinfo = this.sessionStorageService.get<VersionInfo2Response>(
      KEY_VER_INFO
    );
    this.loginRes = this.sessionStorageService.get<LoginResponse>(
      KEY_LOGIN_RES_INFO
    );
  }

  ngOnInit() {}

  ngOnDestroy(): void {}

  onChangePassword(info: {
    currentLoginPw: string;
    newLoginPw: string;
    newConfirmLoginPw: string;
    notValid: () => void;
  }): void {
    if (!info.currentLoginPw || '' === info.currentLoginPw.trim()) {
      // 현재 비밀번호를 입력해주세요
      info.notValid();
      return;
    }
    if (!this.appAuthenticationService.isSameForPassword(info.currentLoginPw)) {
      // "현재 비밀번호가 다릅니다"
      info.notValid();
      return;
    }
    if (!info.newLoginPw || '' === info.newLoginPw.trim()) {
      // 신규 비밀번호를 입력해주세요
      info.notValid();
      return;
    }
    if (-1 < info.newLoginPw.indexOf(this.loginRes.userId)) {
      // "사용자 ID를 비밀번호에 포함할 수 없습니다."
      info.notValid();
      return;
    }
    const phoneNumberOnly = !!this.loginRes.userInfo.hpNumber
      ? this.loginRes.userInfo.hpNumber.trim().replace(new RegExp('-', 'g'), '')
      : '';

    switch (phoneNumberOnly.length) {
      case 11:
        if (-1 < info.newLoginPw.indexOf(phoneNumberOnly.substr(3, 4))) {
          // "사용자 휴대폰번호를 비밀번호에 포함할 수 없습니다."
          info.notValid();
          return;
        }
        if (-1 < info.newLoginPw.indexOf(phoneNumberOnly.substr(7, 4))) {
          // "사용자 휴대폰번호를 비밀번호에 포함할 수 없습니다."
          info.notValid();
          return;
        }
        break;
      case 10:
        if (-1 < info.newLoginPw.indexOf(phoneNumberOnly.substr(3, 3))) {
          // "사용자 휴대폰번호를 비밀번호에 포함할 수 없습니다."
          info.notValid();
          return;
        }
        if (-1 < info.newLoginPw.indexOf(phoneNumberOnly.substr(6, 4))) {
          // "사용자 휴대폰번호를 비밀번호에 포함할 수 없습니다."
          info.notValid();
          return;
        }
        break;

      default:
        break;
    }

    if (!info.newConfirmLoginPw || '' === info.newConfirmLoginPw.trim()) {
      // "신규 비밀번호 확인을 입력해주세요"
      info.notValid();
      return;
    }
    if (info.newLoginPw !== info.newConfirmLoginPw.trim()) {
      // "신규 비밀번호와 신규 비밀번호 확인이 다릅니다"
      info.notValid();
      return;
    }
    if (info.currentLoginPw === info.newLoginPw.trim()) {
      // "현재 비밀번호와 새로운 비밀번호가 같습니다."
      info.notValid();
      return;
    }
    if (0 < StringUtil.includes(info.newLoginPw, CharactorType.Space)) {
      // "비밀번호에는 공백을 입력할 수 없습니다."
      info.notValid();
      return;
    }
    if (StringUtil.isRepeat(info.newLoginPw, 3)) {
      // "숫자나 문자를 3번이상 반복적으로 사용할 수 없습니다."
      info.notValid();
      return;
    }
    if (
      StringUtil.isIncrements(info.newLoginPw, 3) ||
      StringUtil.isDecrements(info.newLoginPw, 3)
    ) {
      // "연속되는 숫자나 문자를 3번이상 사용할 수 없습니다."
      info.notValid();
      return;
    }

    let combinationCount = 0;
    if (0 < StringUtil.includes(info.newLoginPw, CharactorType.Special)) {
      combinationCount++;
    }
    if (0 < StringUtil.includes(info.newLoginPw, CharactorType.Number)) {
      combinationCount++;
    }
    if (0 < StringUtil.includes(info.newLoginPw, CharactorType.Charactor)) {
      combinationCount++;
    }

    let minPwLen = 0;
    if (2 < combinationCount) {
      minPwLen = 8;
    } else if (2 === combinationCount) {
      minPwLen = 10;
    } else if (2 > combinationCount) {
      // "2종류 이상 조합을 해야 합니다."
      info.notValid();
      return;
    }

    if (info.newLoginPw.length < minPwLen) {
      // "비밀번호는 %d가지가 조합된 경우 %d자를 넘어야 합니다."
      info.notValid();
      return;
    }

    this.store.dispatch(
      ServiceStore.userPasswordSet({
        req: {
          loginId: this.loginRes.userId,
          companyCode: this.loginRes.companyCode,
          oldLoginPw: CryptoJS.enc.Hex.stringify(
            CryptoJS.SHA256(info.currentLoginPw)
          ),
          newLoginPw: CryptoJS.enc.Hex.stringify(
            CryptoJS.SHA256(info.newLoginPw)
          )
        }
      })
    );
    this.dialogRef.close({ choice: true });
  }

  onClickChoice(choice: boolean): void {
    this.dialogRef.close({ choice });
  }
}
